/*
 * This part was hacked by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *  harald@haport.sesam.com
 *  Fido: 2:2448/434
 *
 */

#define INCL_DOSSEMAPHORES
#include <os2.h>

#include <sys\types.h>
#include <netinet\in.h>
#include <sys\socket.h>
#include <netdb.h>

#include <string.h>
#include <stdio.h>

#include "sostream.h"

#include "sos_glob.h"

static sock_array[255];
static int nsocks;
static HMTX lck_sem;

#define MAX_PROCSOCKETS  (sizeof(sock_array) / sizeof(sock_array[0]))

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamInit(void)
{
    int result = -1;
    APIRET rc;

    memset(sock_array, 0, sizeof(sock_array));
    nsocks = 0;

    if((rc = DosCreateMutexSem(NULL, &lck_sem, 0L, FALSE)) != 0) {
        fprintf(stderr, "Fatal error %u creating socket mutex semaphore\n", rc);
    }
/*    else if((result = sock_init()) != 0) {
        fprintf(stderr, "Fatal error: INET.SYS not found\n");
    }
*/

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void SoStreamExit(void)
{
    APIRET rc;

    while(nsocks--)
        close(sock_array[nsocks]);
    if((rc = DosCloseMutexSem(lck_sem)) != 0) {
        fprintf(stderr, "Error %u closing socket mutex semaphore\n", rc);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int can_access(void)
{
    APIRET rc;

    for(;;) {
        if((rc = DosRequestMutexSem(lck_sem, 20000)) == 0)
            break;
        if(rc != 121 && rc != 640) {
            fprintf(stderr, "Error %u accessing socket list\n", rc);
            return(0);
        }
    }
    return(1);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static void done_access(void)
{
    APIRET rc;

    if((rc = DosReleaseMutexSem(lck_sem)) != 0) {
        fprintf(stderr, "Error %u releasing socket list\n", rc);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamOpen(void)
{
    int sock = -1;

    if(can_access()) {
        if(nsocks < MAX_PROCSOCKETS) {
            if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
                perror("Opening socket");
            }
            else {
                sock_array[nsocks++] = sock;
            }
        }
        else {
            fprintf(stderr, "Can't open more than %d sockets\n", MAX_PROCSOCKETS);
        }
        done_access();
    }

    return(sock);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamAccept(int sock, char *host, long *inaddr)
{
    struct sockaddr_in client;
    int namelen;
    struct hostent *hp;
    int csock;

    namelen = sizeof(client);
    if ((csock = accept(sock, (void *) &client, &namelen)) == -1) {
        perror("Accepting connection");
        return (-1);
    }
    else if(can_access()) {
        if(nsocks < MAX_PROCSOCKETS) {
            sock_array[nsocks++] = csock;
        }
        else {
            fprintf(stderr, "Can't open more than %d sockets\n", MAX_PROCSOCKETS);
            close(csock);
            done_access();
            return(-1);
        }
        done_access();
    }

    hp = gethostbyaddr((char *)&client.sin_addr, sizeof(struct in_addr), AF_INET);
    if(host){
//        strcpy(host, hp ? (hp->h_name) : inet_ntoa(client.sin_addr));
        strcpy(host, hp ? (hp->h_name) : inet_ntoa(client.sin_addr));
	}
    if(inaddr)
        memcpy(inaddr, &client.sin_addr, sizeof(long));

    return(csock);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamClose(int sock)
{
    int result = -1;
    int i;

    if(can_access()) {
        for(i = 0; i < nsocks; i++)
            if(sock_array[i] == sock)
                break;
        if(i < nsocks) {
            for(i++; i < nsocks; i++)
                sock_array[i - 1] = sock_array[i];
            nsocks--;
            if((result = close(sock)) == -1) {
                perror("SoStreamClose");
            }
        }
        else {
            fprintf(stderr, "Tried to close unopened socket\n");
        }
        done_access();
    }
    return(result);
}
